/*
 * File  : DTP-UART.c:
 * version : 1.0
 * Date : 2022. 7. 21.
 * Contact : expoeb2@diwell.com 
 * Corporation : DIWELL Electronics Co., Ltd.
 *
 * description : 
 *   DTP-UART example program to read the temperature.  
 *   (using wiringPi Library)
 *
 * Further information:
 *   https://projects.drogon.net/raspberry-pi/wiringpi/
 *   https://www.diwellshop.com/
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY.
 ***********************************************************************
 * Note:
 *   Raspberry Pi 2 Model B v1.1 used for testing.
 *   This code has not been tested on other versions of the Raspberry.
 ***********************************************************************
 * Revision history.
 * 1. 2022.7.21 : First version is released.
 ***********************************************************************
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <wiringPi.h>
#include <wiringSerial.h>
typedef unsigned char	uint8_t;
typedef unsigned short	uint16_t; 
typedef unsigned int	uint32_t;
typedef int		int32_t;
typedef short		int16_t;

#define MAX_BUFFER 8		// RX max buffer size
#define IDLE 0
#define CRC_OK	1
#define	CRC_Err	2

uint16_t RX_BUF[MAX_BUFFER], rx_count;
uint8_t Checksum_cal(uint8_t CNT) ;

int main ()
{	
	uint8_t *ptr, responseStatus=0;
	uint8_t TX_BUF[4]={0x11, 0x03, 0x01, 0x98};
	int16_t Object, Ambient;
	int32_t fd ;
	int32_t count ;
	uint32_t DelayMax = 100; // 100ms : Don't change this value.
	uint32_t Start_ms, request_count=0;
 
	if ((fd = serialOpen ("/dev/ttyAMA0", 19200)) < 0)
	{
		fprintf (stderr, "Unable to open serial device: %s\n", strerror (errno)) ;
		return 1 ;
	}

	if (wiringPiSetup () == -1)
	{
		fprintf (stdout, "Unable to start wiringPi: %s\n", strerror (errno)) ;
		return 1 ;
	}

	delay(200);                   // waiting for Sensor init.

	while(request_count++ < 100)  // repeat 100 times.
  	{
    		fflush (stdout) ;
		ptr = TX_BUF;
		responseStatus = IDLE;
		for(count=0; count<4; count++)
		{
			serialPutchar (fd, *ptr++) ; 
		}

		Start_ms = millis();
		while( millis()-Start_ms < DelayMax ) 	// 100ms timer
    		{   
      			while (serialDataAvail (fd))
      			{ 
				RX_BUF[rx_count] = serialGetchar (fd);
      
				switch(rx_count)
      				{
				case 0:
					if(RX_BUF[0] == 0x16){rx_count++;}
					break;
				case 1:
					if(RX_BUF[1] == 0x04){rx_count++;}
					else{rx_count=0;}
					break;
				case 2 ... 6:
					rx_count++;
					break;
				case 7:
					if(RX_BUF[7] == 0x9C)
					{
						if(Checksum_cal(MAX_BUFFER) == RX_BUF[MAX_BUFFER-2])
						{
							Object = (uint16_t)RX_BUF[2]<<8 | (uint16_t)RX_BUF[3];
							Ambient = (uint16_t)RX_BUF[4]<<8 | (uint16_t)RX_BUF[5];
							
							printf ("%4d- Object: %3.1f  Ambient: %3.1f\n", request_count,(float)Object/10, (float)Ambient/10) ;
							responseStatus = CRC_OK;
						}
						else
						{
							printf ("%4d- Checksum Fail.\n", request_count) ;
							responseStatus = CRC_Err;
						}
      
					}
  					rx_count=0;   
					break;
				default: rx_count=0; break;

      				} // end switch(rx_count)

			} //end while (serialDataAvail (fd))
   		} // end while( millis()-Start_ms < DelayMax )	
   		if(responseStatus == IDLE)
   		{
   			printf("%4d-No response. Check the circuit(rx,tx)\n", request_count);
   		}
 	}
 	printf ("================Stopped================\n") ;
  	return 0 ;
}

uint8_t Checksum_cal(uint8_t CNT)    // CHECKSUM 
{
  uint8_t count, SUM=0;
  for(count=0; count<CNT-2; count++)
  {
     SUM += RX_BUF[count];
  }
  return SUM;
}

